/*
	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["epages.event.sensitiveborders"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["epages.event.sensitiveborders"] = true;
dojo.provide("epages.event.sensitiveborders");
dojo.require("epages.browser");

epages.event.sensitiveBorders = {
	// summary: Enable the user to scroll the page by mouse position.
	// description: With this enabled the user can scroll the page by moving the cursor to the particular border in which direction he wants the page to move.
	// example: epages.event.sensitiveBorders.enable(window);
	//          epages.event.sensitiveBorders.disable();

	_SENSITIVE_WIDTH: 80, // px
	_SCROLL_DELAY: 150, // ms
	_SCROLL_SPEED: 2, // ms
	_SCROLL_AMOUNT: 6, // px

	_topInterval: null,
	_leftInterval: null,
	_rightInterval: null,
	_bottomInterval: null,
	_scrollTimeout: null,

	_mouseMoveHandle: null,
	_localDojo: null,
	_localWindow: null,

	maxScrollRight: 0,
	maxScrollBottom: 0,

	_resetTimers: function() {
		clearInterval(this._topInterval);
		clearInterval(this._leftInterval);
		clearInterval(this._rightInterval);
		clearInterval(this._bottomInterval);
		clearTimeout(this._scrollTimeout);
		this._topInterval = null;
		this._leftInterval = null;
		this._rightInterval = null;
		this._bottomInterval = null;
		this._scrollTimeout = null;
	},

	_checkSensitiveArea: function(evt) {
		var moveLeft = evt.clientX < this._SENSITIVE_WIDTH;
		var moveRight = evt.clientX > this.maxScrollRight - this._SENSITIVE_WIDTH;
		var moveTop = evt.clientY < this._SENSITIVE_WIDTH;
		var moveBottom = evt.clientY > this.maxScrollBottom - this._SENSITIVE_WIDTH;
		if(!(moveLeft || moveRight || moveTop || moveBottom)) {
			this._resetTimers();
		}else if(!this._scrollTimeout) {
			this._scrollTimeout = setTimeout(dojo.hitch(this, function() {
				this._resetTimers();
				if(moveLeft) {
					this._leftInterval = setInterval(dojo.hitch(this, function() {
						this._localWindow.scrollBy("-" + this._SCROLL_AMOUNT, 0);
					}), this._SCROLL_SPEED);
				}
				if(moveRight) {
					this._rightInterval = setInterval(dojo.hitch(this, function() {
						// prevent the browser from scrolling into nirvana (especially if user is currently using d&d)
						if((this._localDojo.body().parentNode.scrollLeft || this._localDojo.body().scrollLeft) <= (this._localDojo.body().offsetWidth - this.maxScrollRight)) {
							this._localWindow.scrollBy(this._SCROLL_AMOUNT, 0);
						}
					}), this._SCROLL_SPEED);
				}
				if(moveTop) {
					this._topInterval = setInterval(dojo.hitch(this, function() {
						this._localWindow.scrollBy(0, "-" + this._SCROLL_AMOUNT);
					}), this._SCROLL_SPEED);
				}
				if(moveBottom) {
					this._bottomInterval = setInterval(dojo.hitch(this, function() {
						// prevent the browser from scrolling into nirvana (especially if user is currently using d&d)
						if((this._localDojo.body().parentNode.scrollTop || this._localDojo.body().scrollTop) <= (this._localDojo.body().offsetHeight - this.maxScrollBottom)) {
							this._localWindow.scrollBy(0, this._SCROLL_AMOUNT);
						}
					}), this._SCROLL_SPEED);
				}
			}), this._SCROLL_DELAY);
		}
	},

	enable: function(windowRef /* window reference (optional) */) {
		this._localWindow = windowRef || window;
		this._localDojo = this._localWindow.dojo;
		var bodyNode = this._localDojo.body();

		this.maxScrollRight = (epages.Browser.engine == "WebKit" ? bodyNode.clientWidth : bodyNode.parentNode.clientWidth);
		this.maxScrollBottom = (epages.Browser.engine == "WebKit" ? bodyNode.clientHeight : bodyNode.parentNode.clientHeight);

		if(!bodyNode.parentNode.className.match("SensitiveBorders")) {
			this._localDojo.addClass(bodyNode.parentNode, "SensitiveBorders");
			this._mouseMoveHandle = this._localDojo.connect(bodyNode, "mousemove", this, "_checkSensitiveArea");
		}
	},

	disable: function() {
		this._resetTimers();
		this._localDojo.disconnect(this._mouseMoveHandle);
		this._localDojo.removeClass(this._localDojo.body().parentNode, "SensitiveBorders");
	}
}

}
