define([
    'testsuite/qunit',
    'jquery',
    'ep',
    'testsuite/qunit-fixture!./paypalpluspaymentwall-test',
    'de_epages/paypalplus/ui/paypalpluspaymentwall'
], function(QUnit, $, ep, fixture, paypalpluspaymentwall) {

  'use strict';
  QUnit.module('de_epages/paypalplus/ui/paypalpluspaymentwall', {
      setup: fixture.init,
      teardown: fixture.destroy
  });


  QUnit.test('[Multi checkout] Create payment successful - Build payment wall successful', 3, function(assert) {
    var isMobile = false;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 1
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'none', "Error div is not shown.")
    var SubmitButton = $('.SubmitButton');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.ok(SubmitButtonDisabled == typeof undefined || SubmitButtonDisabled === false, "Submit button has been re-enabled.");
  });

  QUnit.test('[Mobile checkout] Create payment successful - Build payment wall successful', 3, function(assert) {
    var isMobile = true;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 1
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'none', "Error div is not shown.")
    var SubmitButtonBottom = $('#CheckOut');
    var SubmitButtonBottomDisabled = SubmitButtonBottom.prop('disabled');
    assert.ok(SubmitButtonBottomDisabled == typeof undefined || SubmitButtonBottomDisabled === false, "Bottom submit button has been re-enabled.");
    var SubmitButtonTop = $('#CheckOutTop');
    var SubmitButtonTopDisabled = SubmitButtonTop.prop('disabled');
    assert.ok(SubmitButtonTopDisabled == typeof undefined || SubmitButtonTopDisabled === false, "Top submit button has been re-enabled.");
  });

  QUnit.test('[Classical checkout] Create payment successful - Build payment wall successful', 3, function(assert) {
    var isMobile = false;
    var isClassical = true;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 1
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'none', "Error div is not shown.")
    var SubmitButton = $('.ep-uiInput-button');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.ok(SubmitButtonDisabled == typeof undefined || SubmitButtonDisabled === false, "Submit button has been re-enabled.");
  });

  QUnit.test('[Multi checkout] Create payment successful - Build payment wall error', 3, function(assert) {
    var isMobile = false;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
    var SubmitButton = $('.SubmitButton');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.strictEqual(SubmitButtonDisabled, true, "Submit button has been disabled.");
  });

  QUnit.test('[Multi checkout] Create payment connection failed', 3, function(assert) {
    var isMobile = false;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.reject();

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.");
    var SubmitButton = $('.SubmitButton');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.strictEqual(SubmitButtonDisabled, true, "Submit button has been disabled.");
  });

  QUnit.test('[Multi checkout] Create payment not successful', 2, function(assert) {
    var isMobile = false;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
  });

  QUnit.test('[Mobile checkout] Create payment successful - Build payment wall error', 3, function(assert) {
    var isMobile = true;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
    var SubmitButtonBottom = $('#CheckOut');
    var SubmitButtonBottomDisabled = SubmitButtonBottom.prop('disabled');
    assert.strictEqual(SubmitButtonBottomDisabled, true, "Bottom submit button has been disabled.");
    var SubmitButtonTop = $('#CheckOutTop');
    var SubmitButtonTopDisabled = SubmitButtonTop.prop('disabled');
    assert.strictEqual(SubmitButtonTopDisabled, true, "Top submit button has been disabled.");
  });

  QUnit.test('[Mobile checkout] Create payment connection failed', 3, function(assert) {
    var isMobile = true;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.reject();

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
    var SubmitButtonBottom = $('#CheckOut');
    var SubmitButtonBottomDisabled = SubmitButtonBottom.prop('disabled');
    assert.strictEqual(SubmitButtonBottomDisabled, true, "Bottom submit button has been disabled.");
    var SubmitButtonTop = $('#CheckOutTop');
    var SubmitButtonTopDisabled = SubmitButtonTop.prop('disabled');
    assert.strictEqual(SubmitButtonTopDisabled, true, "Top submit button has been disabled.");
  });

  QUnit.test('[Mobile checkout] Create payment not successful', 3, function(assert) {
    var isMobile = true;
    var isClassical = false;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
    var SubmitButtonBottom = $('#CheckOut');
    var SubmitButtonBottomDisabled = SubmitButtonBottom.prop('disabled');
    assert.strictEqual(SubmitButtonBottomDisabled, true, "Bottom submit button has been disabled.");
    var SubmitButtonTop = $('#CheckOutTop');
    var SubmitButtonTopDisabled = SubmitButtonTop.prop('disabled');
    assert.strictEqual(SubmitButtonTopDisabled, true, "Top submit button has been disabled.");
  });

  QUnit.test('[Classical checkout] Create payment successful - Build payment wall error', 3, function(assert) {
    var isMobile = false;
    var isClassical = true;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    PAYPAL.apps.PPP = function(jsonData) {
      jsonData.enableContinue();
      return;
    }

    ep.ajax = function(o) {

      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 1,
          "wasSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
    var SubmitButton = $('.ep-uiInput-button');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.strictEqual(SubmitButtonDisabled, true, "Submit button has been disabled.");
  });

  QUnit.test('[Classical checkout] Create payment connection failed', 3, function(assert) {
    var isMobile = false;
    var isClassical = true;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.reject();

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.");
    var SubmitButton = $('.ep-uiInput-button');
    var SubmitButtonDisabled = SubmitButton.prop('disabled');
    assert.strictEqual(SubmitButtonDisabled, true, "Submit button has been disabled.");
  });

  QUnit.test('[Classical checkout] Create payment not successful', 2, function(assert) {
    var isMobile = false;
    var isClassical = true;
    var BasketOrOrderID = '12345';
    var PaymentLineItemID = '56789';

    ep.ajax = function(o) {
      var result = $.Deferred();

      result.resolve({
          "WasCreatePaymentSuccessful": 0
      });

      return result;
    };

    paypalpluspaymentwall(isMobile, isClassical, BasketOrOrderID, PaymentLineItemID);
    var PayPalPlusDiv = $('#ppplus');
    assert.ok(!PayPalPlusDiv.hasClass('ep-busy'), "Class ep-busy has been removed.");
    var ErrorDiv = $('#PaymentWallErrorMessage');
    assert.strictEqual(ErrorDiv.css('display'), 'block', "Error div is shown.")
  });
});